// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

enum RegType { REG_INVALID = 0xfa5123f, REG_EXPIRED = 0x451fff, REG_NONCOMMERCIAL = 0xa221f562, REG_COMMERCIAL = 0x33f519dc};

struct PROGRAM_SETTING_INFO
{
	enum PROGRAM_SETTING_TYPE { INTEGER, BOOLEAN, STRING, SECTION };
	CString SettingName;
	CString SettingKey;
	CString DefaultValue;
	CString CurrentValue;
	PROGRAM_SETTING_TYPE nType;
};

#include "ProjectDocument.h"
#include "BuildMessageDialog.h"
#include "SplashScreenDialog.h"
#include "CoolMenu.h"

class CBuildProgressDialog;

class CMainFrame : public CMRCMDIFrameWndSizeDock
{
	DECLARE_DYNAMIC(CMainFrame)
public:
	CCoolMenuManager m_MenuMgr;
	BOOL m_bCancelled;
	void DrainPipe(HANDLE hPipe);
	BOOL RunProgram(CString Filename, CString Parameters, CString Directory, CBuildProgressDialog * pDlg);
	void CancelBuild();
	BOOL Simulate(CString Filename);
	void UpdateProjectConfigBox();
	CComboBox m_CurrentConfig;
	CString GetHash(CString String);
	CString Encrypt(CString String);
	CString Decrypt(CString String);
	RegType CheckRegistration(CString Name, CString Key);
	BOOL Transfer(CString Filename, CString HPName = "");
	void NewProject();
	void WriteSetting(CString KeyName);
	void ChangeSetting(CString KeyName, CString NewValue);
	BOOL GetSettingBool(CString KeyName);
	int GetSettingInt(CString KeyName);
	BOOL FileExists(CString Filename);
	void UpdateTitleFile(CString Filename);
	CString Translate(CString TranslateString);
	CString GetSettingString(CString KeyName);
	void ReadProgramSettings();
	void AddProgramSetting(CString Name, CString Key, CString Default, PROGRAM_SETTING_INFO::PROGRAM_SETTING_TYPE Type);
	BOOL CheckDependancies(CString From, CString To);
	void MakeHPLibrary(CString Filename, CBuildProgressDialog& dlg);
	void UpdateConfigFile(CString Filename);
	void UpdateVisibleFile(CString Filename);
	void WriteNibbleWord(CArray<BYTE, BYTE>& NibbleArray, int nWord, BOOL bUpdateSize = TRUE);
	DWORD m_nNibbleOffset;
	void WriteNibbles(CArray<BYTE, BYTE>& NibbleArray, void * Buffer, int Length);
	BOOL MakeHPDirectory(CString FileName, CBuildProgressDialog& dlg);
	CDocTemplate* m_pSourceTemplate;
	CDocTemplate* m_pProjectTemplate;
	HANDLE m_hCancelBuild;
	BOOL m_bBuilding;
	BOOL Compile(CString Filename, CBuildProgressDialog * pDlg = NULL, BOOL bCheckIfNeeded = FALSE);
	CBuildMessageDialog* m_pBuildMessageDialog;
	CProjectDocument* m_pCurrentProject;
	CMainFrame();

// Attributes
public:
	CArray<PROGRAM_SETTING_INFO, PROGRAM_SETTING_INFO> m_ProgramSettingList;

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMainFrame)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CMainFrame();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

public:  // control bar embedded members
	CStatusBar  m_wndStatusBar;
	CToolBar    m_wndMainToolBar, m_wndProjectToolBar;
	CBuildMessageDialog	m_wndMessageDialogBar;
	CSplashScreenDialog* m_pSplashScreenDialog;

// Generated message map functions
protected:
	//{{AFX_MSG(CMainFrame)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnProjectCloseproject();
	afx_msg void OnProjectNewproject();
	afx_msg void OnProjectOpenproject();
	afx_msg void OnProjectSaveproject();
	afx_msg void OnUpdateProjectCloseproject(CCmdUI* pCmdUI);
	afx_msg void OnUpdateProjectNewproject(CCmdUI* pCmdUI);
	afx_msg void OnUpdateProjectOpenproject(CCmdUI* pCmdUI);
	afx_msg void OnUpdateProjectSaveproject(CCmdUI* pCmdUI);
	afx_msg void OnFileNew();
	afx_msg void OnUpdateFileNew(CCmdUI* pCmdUI);
	afx_msg void OnUpdateProjectProjectsettings(CCmdUI* pCmdUI);
	afx_msg void OnProjectProjectsettings();
	afx_msg void OnFileSaveall();
	afx_msg void OnBuildBuildproject();
	afx_msg void OnUpdateBuildBuildproject(CCmdUI* pCmdUI);
	afx_msg void OnViewOptions();
	afx_msg void OnHelpTipoftheday();
	afx_msg void OnBuildTransferproject();
	afx_msg void OnUpdateBuildTransferproject(CCmdUI* pCmdUI);
	afx_msg void OnFileTransferfile();
	afx_msg void OnUpdateBuildCancelbuild(CCmdUI* pCmdUI);
	afx_msg void OnBuildCancelbuild();
	afx_msg void OnUpdateProjectAddfilefromdisk(CCmdUI* pCmdUI);
	afx_msg void OnProjectAddfilefromdisk();
	afx_msg void OnUpdateProjectCurrentconfig(CCmdUI* pCmdUI);
	afx_msg void OnSelchangeCurrentConfig();
	afx_msg void OnUpdateBuildSimulateproject(CCmdUI* pCmdUI);
	afx_msg void OnBuildSimulateproject();
	afx_msg void OnEditTemplatesEdit();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
