#include "stdafx.h"
#include "resource.h"
#include "TipDlg.h"
#include "MainFrm.h"
#include "HPIDE.h"

#include <winreg.h>
#include <sys\stat.h>
#include <sys\types.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTipDlg dialog

#define MAX_BUFLEN 1000

CString Tips[] =
{
	"Welcome to HP-IDE!  You can disable these tips on startup with the "
	"checkbox below.  If you've disabled them, you can always reenable them "
	"from the Tip of the Day option under Help or the Program Options dialog.  "
	"There are lots of good tips in here, so you should probably leave "
	"them on for at least a little while.  \n\nEnjoy!"
	,
	"The official HP-IDE homepage is currently at:\n\n"
	"http://www.acs.ucalgary.ca/~mmastrac/hp-ide\n\n"
	"Here, you can find all sorts of neat things, like the HP-IDE mailing "
	"list information, a wishlist and program news."
	,
	"The order of the objects in a directory or library is the same as their "
	"order in the project.  Use the file up/down commands on the project "
	"toolbar to shift them into the order you want them to go."
	,
	"You can use the HP's special characters in the library and object names "
	"by using the escape codes on page 27-16 of your HP manual."
	,
	"RPL stands for either ROM-based Procedural Language or Reverse Polish "
	"Lisp (preferred)."
	,
	"Using multiple project configurations is a good way to maintain multiple "
	"versions of a program that have different features.  You can create "
	"a regular version and a lite version, for example."
	,
	"Simulating projects with EMU48 is a lot easier on your batteries than "
	"transferring the program over and over.  You need to download the version "
	"specifically designed to work with HP-IDE, however (from the HP-IDE website)."
	,
	"If your directory looks odd on your HP48 or your library won't compile/attach "
	"properly, you may have an invalid object somewhere in it.  Upload the individual "  
	"objects to the HP48 and see which ones are invalid or fail to transfer properly."
	,
	"To open multiple source files at a time, use the File/Open command, but instead "
	"drag the files you want out of the dialog and into the main window.  You can also "
	"drag and drop from Windows Explorer with both projects and source files."
	,
	"Use source templates to make repetitive tasks (like NIBASC headers and "
	"parameterized outer loops) easier.  A few templates are included to start you off, "
	"but you can edit them to suit your needs."
	,
	"If your assembler opcode isn't indented, SASM will treat it as a label "
	"and your source may not work/compile correctly."
	,
	"If you don't have the \"NIBASC /HPHP48-x/\" header in your source files you "
	"won't be able to upload them directly, but you can still build a valid "
	"library or directory (you'll get a warning, however)."
	,
	"You can't close a project by using the close box on the caption bar.  "
	"Use the Close Project toolbar button or menu option instead."
	,
	"To override the default config file generation, just delete ProjectConfig.s "
	"from the project and then add your own file (with a different name) and "
	"make its compiled name '$CONFIG'."
	,
	"You can include header files in your project for convenience if you set their "
	"compile flag to 'No'."
	,
	"You can specify library variables to be stored in user memory with the "
	"$VARS objects.  See the USRLIB documentation for more information."
};

CTipDlg::CTipDlg(CWnd* pParent /*=NULL*/)
	: CDialog(IDD_TIPOFTHEDAY, pParent)
{
	//{{AFX_DATA_INIT(CTipDlg)
	m_bStartup = TRUE;
	//}}AFX_DATA_INIT

	GetNextTipString(m_strTip);
}

CTipDlg::~CTipDlg()
{
}
        
void CTipDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTipDlg)
	DDX_Check(pDX, IDC_STARTUP, m_bStartup);
	DDX_Text(pDX, IDC_TIP_TIPTEXT, m_strTip);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CTipDlg, CDialog)
	//{{AFX_MSG_MAP(CTipDlg)
	ON_BN_CLICKED(IDC_NEXTTIP, OnNextTip)
	ON_WM_CTLCOLOR()
	ON_WM_PAINT()
	ON_BN_CLICKED(IDC_STARTUP, OnStartup)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTipDlg message handlers

void CTipDlg::OnNextTip()
{
	GetNextTipString(m_strTip);
	UpdateData(FALSE);
}

void CTipDlg::GetNextTipString(CString& strNext)
{
	int nTip = GetMainWnd()->GetSettingInt("TipNum");
	if (nTip >= sizeof(Tips) / sizeof(CString))
		nTip = 0;
	strNext = Tips[nTip];
	nTip++;
	CString TipNum;
	TipNum.Format("%i", nTip);
	GetMainWnd()->ChangeSetting("TipNum", TipNum);
	GetMainWnd()->WriteSetting("TipNum");
}

HBRUSH CTipDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	if (pWnd->GetDlgCtrlID() == IDC_TIP_TIPTEXT)
		return (HBRUSH)GetStockObject(WHITE_BRUSH);

	return CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
}

void CTipDlg::OnOK()
{
	CDialog::OnOK();
}

BOOL CTipDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	m_bStartup = GetMainWnd()->GetSettingBool("ShowTips");
	UpdateData(FALSE);
	GetDlgItem(IDC_TIP_TIPTEXT)->SetFont(&((CHPIDEApp*)AfxGetApp())->m_TipFont, TRUE);

	return TRUE;  // return TRUE unless you set the focus to a control
}

void CTipDlg::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	// Get paint area for the big static control
	CWnd* pStatic = GetDlgItem(IDC_BIGFRAME);
	CRect rect;
	pStatic->GetWindowRect(&rect);
	ScreenToClient(&rect);

	int nFarRight = rect.right;

	// Paint the background white.
	CBrush brush;
	brush.CreateStockObject(WHITE_BRUSH);
	dc.FillRect(rect, &brush);

	pStatic = GetDlgItem(IDC_BULB);
	pStatic->GetWindowRect(&rect);
	ScreenToClient(&rect);

	// Load bitmap and get dimensions of the bitmap
	CBitmap bmp;
	bmp.LoadBitmap(IDB_LIGHTBULB);
	BITMAP bmpInfo;
	bmp.GetBitmap(&bmpInfo);

	// Draw bitmap in top corner and validate only top portion of window
	CDC dcTmp;
	dcTmp.CreateCompatibleDC(&dc);
	dcTmp.SelectObject(&bmp);
	rect.bottom = bmpInfo.bmHeight + rect.top;
	dc.BitBlt(rect.left, rect.top, rect.Width(), rect.Height(), 
		&dcTmp, 0, 0, SRCCOPY);

	// Draw out "Did you know..." message next to the bitmap
	CString strMessage;
	strMessage.LoadString(IDS_DIDYOUKNOW);
	rect.left += bmpInfo.bmWidth;
	rect.right = nFarRight;
	dc.DrawText(strMessage, rect, DT_VCENTER | DT_SINGLELINE);

	// Do not call CDialog::OnPaint() for painting messages
}

void CTipDlg::OnStartup() 
{
	UpdateData(TRUE);
	GetMainWnd()->ChangeSetting("ShowTips", m_bStartup ? "Yes" : "No");
	GetMainWnd()->WriteSetting("ShowTips");
}
