// ProgramSettingsDialog.cpp : implementation file
//

#include "stdafx.h"
#include "HPIDE.h"
#include "ProgramSettingsDialog.h"
#include "EditProgramSettingDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CProgramSettingsDialog dialog


CProgramSettingsDialog::CProgramSettingsDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CProgramSettingsDialog::IDD, pParent)
{
	//{{AFX_DATA_INIT(CProgramSettingsDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CProgramSettingsDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CProgramSettingsDialog)
	DDX_Control(pDX, IDC_SETTINGS_SETTINGSBOX, m_ProgramSettings);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CProgramSettingsDialog, CDialog)
	//{{AFX_MSG_MAP(CProgramSettingsDialog)
	ON_NOTIFY(NM_DBLCLK, IDC_SETTINGS_SETTINGSBOX, OnDblclkSettingsSettingsbox)
	ON_BN_CLICKED(IDC_SETTINGS_DEFAULT, OnSettingsDefault)
	ON_BN_CLICKED(IDC_SETTINGS_COPYTOCLIPBOARD, OnSettingsCopytoclipboard)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CProgramSettingsDialog message handlers

BOOL CProgramSettingsDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();
	CRect r;
	m_ProgramSettings.GetClientRect(&r);
	int nWidth = r.Width() - GetSystemMetrics(SM_CXHSCROLL);
	m_ProgramSettings.InsertColumn(0, _T("Name"), LVCFMT_LEFT, nWidth * 30 / 100);
	nWidth -= nWidth * 30 / 100;
	m_ProgramSettings.InsertColumn(1, _T("Value"), LVCFMT_LEFT, nWidth);
	
	CArray<PROGRAM_SETTING_INFO, PROGRAM_SETTING_INFO>* pSettings =
		&((CMainFrame*)AfxGetMainWnd())->m_ProgramSettingList;
	int nSpaceCount = 0;
	for (int i = 0; i < pSettings->GetSize(); i++)
	{
		if ((*pSettings)[i].nType == PROGRAM_SETTING_INFO::SECTION)
		{
			int nIndex = i + nSpaceCount;
			m_ProgramSettings.InsertItem(nIndex, "");
			m_ProgramSettings.SetItemData(nIndex, 0xffffffff);
			if (i != 0) 
			{
				nIndex++;
				nSpaceCount++;
			}
			m_ProgramSettings.InsertItem(nIndex, (*pSettings)[i].SettingName);
			m_ProgramSettings.SetItemText(nIndex, 1, (*pSettings)[i].CurrentValue);
			m_ProgramSettings.SetItemData(nIndex, 0xffffffff);
		}
		else
		{
			int nIndex = i + nSpaceCount;
			m_ProgramSettings.InsertItem(nIndex, (*pSettings)[i].SettingName);
			m_ProgramSettings.SetItemText(nIndex, 1, (*pSettings)[i].CurrentValue);
			m_ProgramSettings.SetItemData(nIndex, i);
		}
	}
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CProgramSettingsDialog::OnDblclkSettingsSettingsbox(NMHDR* pNMHDR, LRESULT* pResult) 
{
	int nIndex = -1;
	for (int i = 0; i < m_ProgramSettings.GetItemCount(); i++)
	{
		if (m_ProgramSettings.GetItemState(i, LVIS_SELECTED))
		{
			nIndex = i;
			break;
		}
	}
	if (nIndex != -1 && m_ProgramSettings.GetItemData(nIndex) != 0xffffffff)
	{
		int nRealIndex = m_ProgramSettings.GetItemData(nIndex);
		if (GetMainWnd()->m_ProgramSettingList[nRealIndex].nType == PROGRAM_SETTING_INFO::BOOLEAN)
		{
			CString Key = GetMainWnd()->m_ProgramSettingList[nRealIndex].SettingKey;

			if (GetMainWnd()->GetSettingBool(Key))
				GetMainWnd()->ChangeSetting(Key, "No");
			else
				GetMainWnd()->ChangeSetting(Key, "Yes");
			m_ProgramSettings.SetItemText(nIndex, 1, GetMainWnd()->GetSettingString(Key));
		}
		else
		{
			CEditProgramSettingDialog dlg;
			dlg.m_OldValue = dlg.m_NewValue = 
				m_ProgramSettings.GetItemText(nIndex, 1);
			if (dlg.DoModal() == IDOK)
			{
				m_ProgramSettings.SetItemText(nIndex, 1, dlg.m_NewValue);
				GetMainWnd()->m_ProgramSettingList[nRealIndex].CurrentValue = dlg.m_NewValue;
			}
		}
	}
	*pResult = 0;
}

void CProgramSettingsDialog::OnOK() 
{
	HKEY hMainKey;
	if (RegCreateKey(HKEY_CURRENT_USER, "Software\\HP-IDE\\Options", &hMainKey) == ERROR_SUCCESS)
	{
		for (int i = 0; i < GetMainWnd()->m_ProgramSettingList.GetSize(); i++)
		{
			PROGRAM_SETTING_INFO Info = GetMainWnd()->m_ProgramSettingList[i];
			if (Info.nType != PROGRAM_SETTING_INFO::SECTION)
			{
				HKEY hItemKey;
				if (RegCreateKey(hMainKey, Info.SettingKey, &hItemKey) == ERROR_SUCCESS)
				{
					CString KeyData = Info.CurrentValue;
					RegSetValue(hItemKey, NULL, REG_SZ, KeyData, KeyData.GetLength()) == ERROR_SUCCESS;
					RegCloseKey(hItemKey);
				}
			}
		}
		RegCloseKey(hMainKey);
	}	
	
	CDialog::OnOK();
}

void CProgramSettingsDialog::OnSettingsDefault() 
{
	int nIndex = -1;
	for (int i = 0; i < m_ProgramSettings.GetItemCount(); i++)
	{
		if (m_ProgramSettings.GetItemState(i, LVIS_SELECTED))
		{
			nIndex = i;
			break;
		}
	}
	if (nIndex != -1 && m_ProgramSettings.GetItemData(nIndex) != 0xffffffff)
	{
		int nRealIndex = m_ProgramSettings.GetItemData(nIndex);
		CString Default = GetMainWnd()->m_ProgramSettingList[nRealIndex].DefaultValue;
		CString Message = "Restore the setting \"";
		Message += m_ProgramSettings.GetItemText(nIndex, 0);
		Message += "\" to its default value of \"";
		Message += Default + "\"?";
		if (MessageBox(Message, "Confirm", MB_YESNO | MB_ICONQUESTION) == IDNO)
			return;
		m_ProgramSettings.SetItemText(nIndex, 1, Default);
		GetMainWnd()->m_ProgramSettingList[nRealIndex].CurrentValue = Default;
	}
}

void CProgramSettingsDialog::OnSettingsCopytoclipboard() 
{
	if (!OpenClipboard())
	{
		AfxMessageBox(IDS_COULDNTOPENCLIPBOARD);
		return;
	}

	EmptyClipboard();

	CString SettingsList = "HP-IDE Settings:\r\n";

	CArray<PROGRAM_SETTING_INFO, PROGRAM_SETTING_INFO>* pSettings =
		&((CMainFrame*)AfxGetMainWnd())->m_ProgramSettingList;
	int nSpaceCount = 0;
	for (int i = 0; i < pSettings->GetSize(); i++)
	{
		if ((*pSettings)[i].nType == PROGRAM_SETTING_INFO::SECTION)
		{
			SettingsList += "\r\n" + (*pSettings)[i].SettingName + "\r\n----------\r\n";
		}
		else
		{
			if ((*pSettings)[i].SettingName.Find("Registration") != -1)
				SettingsList += (*pSettings)[i].SettingName + ": " + "not copied\r\n";
			else
				SettingsList += (*pSettings)[i].SettingName + ": " + (*pSettings)[i].CurrentValue + "\r\n";
		}
	}

	HANDLE hMem = GlobalAlloc(GPTR, SettingsList.GetLength() + 1);
	CopyMemory(hMem, SettingsList, SettingsList.GetLength() + 1);
	SetClipboardData(CF_TEXT, hMem);

	CloseClipboard();	
}
