// ProjectDocument.cpp : implementation file
//

#include "stdafx.h"
#include "HPIDE.h"
#include "ProjectDocument.h"
#include "ProjectEditView.h"
#include "EditFilenameDialog.h"
#include "EditFilePropertiesDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CProjectDocument

IMPLEMENT_DYNCREATE(CProjectDocument, CDocument)

CProjectDocument::CProjectDocument()
{
}

BOOL CProjectDocument::OnNewDocument()
{
	m_ProjectType = "Directory";
	GetCurrentDirectory(MAX_PATH, m_ProjectDirectory.GetBuffer(MAX_PATH));
	m_ProjectDirectory.ReleaseBuffer();
	m_FileList.SetSize(0);
	m_nROMID = 1000;
	m_ProjectConfigList.SetSize(1);
	m_nCurrentConfig = 0;
	CString s;
	s.LoadString(IDS_DEFAULTCONFIG);
	m_ProjectConfigList[0].ConfigName = s;
	s.LoadString(IDS_DEFAULTPROJECTFILE);
	m_ProjectConfigList[0].BaseFilename = s;
	s.LoadString(IDS_DEFAULTPROJECTTITLE);
	m_ProjectConfigList[0].LibraryTitle = s;
	UpdateListControl();
	if (!CDocument::OnNewDocument())
		return FALSE;
	return TRUE;
}

CProjectDocument::~CProjectDocument()
{
}

BEGIN_MESSAGE_MAP(CProjectDocument, CDocument)
	//{{AFX_MSG_MAP(CProjectDocument)
	ON_UPDATE_COMMAND_UI(ID_FILE_CLOSE, OnUpdateFileClose)
	ON_COMMAND(ID_PROJECT_TOGGLECOMPILE, OnProjectTogglecompile)
	ON_UPDATE_COMMAND_UI(ID_PROJECT_TOGGLECOMPILE, OnUpdateProjectTogglecompile)
	ON_COMMAND(ID_PROJECT_TOGGLEFILEVISIBILITY, OnProjectTogglefilevisibility)
	ON_UPDATE_COMMAND_UI(ID_PROJECT_TOGGLEFILEVISIBILITY, OnUpdateProjectTogglefilevisibility)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
	ON_UPDATE_COMMAND_UI(ID_PROJECT_EDITFILESETTINGS, OnUpdateProjectEditfilesettings)
	ON_COMMAND(ID_PROJECT_EDITFILESETTINGS, OnProjectEditfilesettings)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CProjectDocument diagnostics

#ifdef _DEBUG
void CProjectDocument::AssertValid() const
{
	CDocument::AssertValid();
}

void CProjectDocument::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CProjectDocument serialization

void CProjectDocument::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		int nVersion = 1;
		ar << nVersion;
		ar << m_ProjectType;
		ar << m_nROMID;
		ar << m_nCurrentConfig;
		ar << m_ProjectConfigList.GetSize();
		for (int i = 0; i < m_ProjectConfigList.GetSize(); i++)
		{
			ar << m_ProjectConfigList[i].ConfigName;
			ar << m_ProjectConfigList[i].BaseFilename;
			ar << m_ProjectConfigList[i].LibraryTitle;
		}
		ar << m_FileList.GetSize();
		for (int j = 0; j < m_FileList.GetSize(); j++)
		{
			ar << m_FileList[j].Filename;
			ar << m_FileList[j].bCompile;
			ar << m_FileList[j].bVisible;
			ar << m_FileList[j].HPName;
			ar << m_FileList[j].bInclude;
			ar << m_FileList[j].ConfigList;
		}
	}
	else
	{
		int i;
		CString s;
		int nVersion;
		ar >> nVersion;
		int nSize;
		switch (nVersion)
		{
		case 0:
			m_ProjectConfigList.SetSize(1);
			m_nCurrentConfig = 0;
			s.LoadString(IDS_DEFAULTCONFIG);
			m_ProjectConfigList[0].ConfigName = s;
			ar >> m_ProjectType;
			ar >> nSize;
			ar >> m_ProjectConfigList[0].BaseFilename;
			ar >> m_nROMID;
			ar >> m_ProjectConfigList[0].LibraryTitle;
			m_FileList.SetSize(nSize);
			for (i = 0; i < nSize; i++)
			{
				ar >> m_FileList[i].Filename;
				ar >> m_FileList[i].bCompile;
				ar >> m_FileList[i].bVisible;
				ar >> m_FileList[i].HPName;
				m_FileList[i].bInclude = FALSE;
				m_FileList[i].ConfigList = "";
			}
			break;
		case 1:
			ar >> m_ProjectType;
			ar >> m_nROMID;
			ar >> m_nCurrentConfig;
			ar >> nSize;
			m_ProjectConfigList.SetSize(nSize);
			for (i = 0; i < nSize; i++)
			{
				ar >> m_ProjectConfigList[i].ConfigName;
				ar >> m_ProjectConfigList[i].BaseFilename;
				ar >> m_ProjectConfigList[i].LibraryTitle;
			}			
			ar >> nSize;
			m_FileList.SetSize(nSize);
			for (i = 0; i < nSize; i++)
			{
				ar >> m_FileList[i].Filename;
				ar >> m_FileList[i].bCompile;
				ar >> m_FileList[i].bVisible;
				ar >> m_FileList[i].HPName;
				ar >> m_FileList[i].bInclude;
				ar >> m_FileList[i].ConfigList;
			}
			break;
		default:
			AfxMessageBox(IDS_PROJECTVERSIONNOTSUPPORTED);
			AfxThrowArchiveException(CArchiveException::badSchema);
		}
		UpdateListControl();
	}
}

/////////////////////////////////////////////////////////////////////////////
// CProjectDocument commands

BOOL CProjectDocument::CanCloseFrame(CFrameWnd* pFrame) 
{
	return FALSE;	
}

void CProjectDocument::UpdateListControl()
{
	POSITION pos = GetFirstViewPosition();
	CProjectEditView* pView = (CProjectEditView*)GetNextView(pos);
	CListCtrl& ListCtrl = pView->GetListCtrl();
	
	ListCtrl.DeleteAllItems();
	for (int i = 0; i < m_FileList.GetSize(); i++)
	{
		ListCtrl.InsertItem(i, m_FileList[i].Filename);
		ListCtrl.SetItemText(i, 1, (m_FileList[i].bVisible ? "Yes" : "No"));
		ListCtrl.SetItemText(i, 2, (m_FileList[i].bCompile ? "Yes" : "No"));
		ListCtrl.SetItemText(i, 3, m_FileList[i].HPName);

		BOOL bInclude = FileInCurrentConfig(i);
		ListCtrl.SetItem(i, 0, LVIF_IMAGE, NULL, bInclude ? 1 : 0, 0, 0, 0);
	}
}

void CProjectDocument::UpdateListControlFlags()
{
	POSITION pos = GetFirstViewPosition();
	CProjectEditView* pView = (CProjectEditView*)GetNextView(pos);
	CListCtrl& ListCtrl = pView->GetListCtrl();
	
	for (int i = 0; i < m_FileList.GetSize(); i++)
	{
		BOOL bInclude = FileInCurrentConfig(i);
		ListCtrl.SetItem(i, 0, LVIF_IMAGE, NULL, bInclude ? 1 : 0, 0, 0, 0);
	}
}

void CProjectDocument::UpdateListControlItem(int nIndex)
{
	POSITION pos = GetFirstViewPosition();
	CProjectEditView* pView = (CProjectEditView*)GetNextView(pos);
	CListCtrl& ListCtrl = pView->GetListCtrl();
	
	ListCtrl.SetItemText(nIndex, 0, m_FileList[nIndex].Filename);
	ListCtrl.SetItemText(nIndex, 1, (m_FileList[nIndex].bVisible ? "Yes" : "No"));
	ListCtrl.SetItemText(nIndex, 2, (m_FileList[nIndex].bCompile ? "Yes" : "No"));
	ListCtrl.SetItemText(nIndex, 3, m_FileList[nIndex].HPName);
	BOOL bInclude = FileInCurrentConfig(nIndex);
	ListCtrl.SetItem(nIndex, 0, LVIF_IMAGE, NULL, bInclude ? 1 : 0, 0, 0, 0);
}

void CProjectDocument::OnFileSaveWrapper()
{
	OnFileSave();
}

void CProjectDocument::OnUpdateFileClose(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);	
}

BOOL CProjectDocument::AddFileToProject(CString Filename, CString HPName /* = "" */)
{
	if (GetPathName().IsEmpty())
		return FALSE;

	CString FilePath;
	char* pFilename;
	char* pPath = FilePath.GetBuffer(MAX_PATH);
	GetFullPathName(Filename, MAX_PATH, pPath, &pFilename);
	CString FileBase = pFilename;
	FilePath.ReleaseBuffer();
	FilePath = FilePath.Left(pFilename - pPath);
	
	CString ProjectFile = GetPathName();
	CString ProjectPath;
	pPath = ProjectPath.GetBuffer(MAX_PATH);
	GetFullPathName(ProjectFile, MAX_PATH, pPath, &pFilename);
	ProjectPath.ReleaseBuffer();
	ProjectPath = ProjectPath.Left(pFilename - pPath);

	if (FilePath.Left(ProjectPath.GetLength()) == ProjectPath)
		Filename = Filename.Right(Filename.GetLength() - ProjectPath.GetLength());

	if (FileBase.Right(2) == (".s"))
		FileBase = FileBase.Left(FileBase.GetLength() - 2);
	if (FileBase.Right(2) == (".S"))
		FileBase = FileBase.Left(FileBase.GetLength() - 2);

	PROJECT_FILE_INFO Info;
	Info.Filename = Filename;
	Info.bVisible = TRUE;
	Info.bCompile = TRUE;
	Info.bInclude = FALSE;

	if (HPName.IsEmpty())
		Info.HPName = FileBase;
	else 
		Info.HPName = HPName;
	m_FileList.Add(Info);

	POSITION pos = GetFirstViewPosition();
	CProjectEditView* pView = (CProjectEditView*)GetNextView(pos);

	CListCtrl& ListCtrl = pView->GetListCtrl();
	int nItem = ListCtrl.GetItemCount();
	ListCtrl.InsertItem(nItem, Info.Filename);
	UpdateListControlItem(nItem);

	SetModifiedFlag(TRUE);

	return TRUE;
}

BOOL CProjectDocument::FileInProject(CString Filename)
{
	for (int i = 0; i < m_FileList.GetSize(); i++)
	{
		if (m_FileList[i].Filename == Filename)
			return TRUE;
	}
	return FALSE;
}

BOOL CProjectDocument::GetFileInfo(CString Filename, PROJECT_FILE_INFO& Info)
{
	for (int i = 0; i < m_FileList.GetSize(); i++)
	{
		if (m_FileList[i].Filename == Filename)
		{
			Info = m_FileList[i];
			return TRUE;
		}
	}
	return FALSE;
}

BOOL CProjectDocument::SetFileInfo(CString Filename, PROJECT_FILE_INFO& Info)
{
	for (int i = 0; i < m_FileList.GetSize(); i++)
	{
		if (m_FileList[i].Filename == Filename)
		{
			m_FileList[i].bCompile = Info.bCompile;
			m_FileList[i].bVisible = Info.bVisible;
			SetModifiedFlag(TRUE);
			UpdateListControlItem(i);	
			return TRUE;
		}
	}
	return FALSE;
}

void CProjectDocument::OnProjectTogglecompile() 
{
	BOOL bVisible, bCompile;
	GetSelectedFileState(bVisible, bCompile);
	
	POSITION pos = GetFirstViewPosition();
	CProjectEditView* pView = (CProjectEditView*)GetNextView(pos);

	CListCtrl& ListCtrl = pView->GetListCtrl();

	for (int i = 0; i < ListCtrl.GetItemCount(); i++)
	{
		if (ListCtrl.GetItemState(i, LVIS_SELECTED))
		{
			m_FileList[i].bCompile = !bCompile;
			UpdateListControlItem(i);
		}
	}
}

void CProjectDocument::OnUpdateProjectTogglecompile(CCmdUI* pCmdUI) 
{
	BOOL bVisible, bCompile;
	pCmdUI->Enable(GetSelectedFileState(bVisible, bCompile));
	pCmdUI->SetCheck(bCompile);
}

void CProjectDocument::OnProjectTogglefilevisibility() 
{
	BOOL bVisible, bCompile;
	GetSelectedFileState(bVisible, bCompile);
	
	POSITION pos = GetFirstViewPosition();
	CProjectEditView* pView = (CProjectEditView*)GetNextView(pos);

	CListCtrl& ListCtrl = pView->GetListCtrl();

	for (int i = 0; i < ListCtrl.GetItemCount(); i++)
	{
		if (ListCtrl.GetItemState(i, LVIS_SELECTED))
		{
			m_FileList[i].bVisible = !bVisible;
			UpdateListControlItem(i);
		}
	}
}

void CProjectDocument::OnUpdateProjectTogglefilevisibility(CCmdUI* pCmdUI) 
{
	BOOL bVisible, bCompile;
	pCmdUI->Enable(GetSelectedFileState(bVisible, bCompile));
	pCmdUI->SetCheck(bVisible);
}

BOOL CProjectDocument::GetSelectedFileState(BOOL& bVisible, BOOL& bCompile)
{
	BOOL bVisibleNo = FALSE;
	BOOL bCompileNo = FALSE;
	BOOL bSelected = FALSE;
	POSITION pos = GetFirstViewPosition();
	CProjectEditView* pView = (CProjectEditView*)GetNextView(pos);

	CListCtrl& ListCtrl = pView->GetListCtrl();

	for (int i = 0; i < ListCtrl.GetItemCount(); i++)
	{
		if (ListCtrl.GetItemState(i, LVIS_SELECTED))
		{
			bSelected = TRUE;
			if (ListCtrl.GetItemText(i, 1) == "No")
				bVisibleNo = TRUE;
			if (ListCtrl.GetItemText(i, 2) == "No")
				bCompileNo = TRUE;
		}
	}

	if (bSelected)
	{
		bVisible = !bVisibleNo;
		bCompile = !bCompileNo;
	}
	else
	{
		bVisible = bCompile = FALSE;
	}
	
	return bSelected;
}


BOOL CProjectDocument::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (GetMainWnd()->m_pCurrentProject)
	{
		AfxGetMainWnd()->MessageBox("Only one project can be open at a time!", "Open Error");
		return FALSE;
	}

	if (!CDocument::OnOpenDocument(lpszPathName))
		return FALSE;
	
	GetMainWnd()->m_pCurrentProject = this;
	GetMainWnd()->UpdateProjectConfigBox();
	GetMainWnd()->SendMessage(WM_COMMAND, ID_WINDOW_CASCADE, 0);
	return TRUE;
}

void CProjectDocument::OnUpdateFileSaveAs(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(FALSE);
}

CString CProjectDocument::GetProjectDir()
{
	CString ProjectDir;
	char* pFilename;
	char* pPath = ProjectDir.GetBuffer(MAX_PATH);
	GetFullPathName(GetPathName(), MAX_PATH, pPath, &pFilename);
	ProjectDir.ReleaseBuffer();
	ProjectDir = ProjectDir.Left(pFilename - pPath);

	return ProjectDir;
}

void CProjectDocument::MoveItem(int OldIndex, int NewIndex)
{
	PROJECT_FILE_INFO Info = m_FileList[OldIndex];
	m_FileList.RemoveAt(OldIndex);
	m_FileList.InsertAt(NewIndex, Info);
}

CString CProjectDocument::GetBaseName()
{
	return m_ProjectConfigList[m_nCurrentConfig].BaseFilename;
}

CString CProjectDocument::GetLibraryTitle()
{
	return m_ProjectConfigList[m_nCurrentConfig].LibraryTitle;
}

CString CProjectDocument::GetCurrentConfigName()
{
	return m_ProjectConfigList[m_nCurrentConfig].ConfigName;
}

int CProjectDocument::GetCurrentConfig()
{
	return m_nCurrentConfig;
}

void CProjectDocument::OnUpdateProjectEditfilesettings(CCmdUI* pCmdUI) 
{
	POSITION pos = GetFirstViewPosition();
	CProjectEditView* pView = (CProjectEditView*)GetNextView(pos);

	CListCtrl& ListCtrl = pView->GetListCtrl();

	pCmdUI->Enable(ListCtrl.GetSelectedCount() == 1);
}

void CProjectDocument::OnProjectEditfilesettings() 
{
	CEditFilePropertiesDialog dlg;

	POSITION pos = GetFirstViewPosition();
	CProjectEditView* pView = (CProjectEditView*)GetNextView(pos);

	CListCtrl& ListCtrl = pView->GetListCtrl();
	int nIndex = ListCtrl.GetNextItem(-1, LVNI_SELECTED);

	ASSERT(nIndex != -1);
	if (nIndex == -1)
		return;
	PROJECT_FILE_INFO Info = m_FileList[nIndex];

	dlg.m_bCompile = Info.bCompile;
	dlg.m_bVisible = Info.bVisible;

	dlg.m_nInclude = Info.bInclude ? 1 : 0;
	dlg.m_ConfigList = Info.ConfigList;
	dlg.m_RelativePath = Info.Filename;
	dlg.m_CompiledName = Info.HPName;
	if (dlg.DoModal() == IDOK)
	{
		m_FileList[nIndex].bCompile = dlg.m_bCompile;
		m_FileList[nIndex].bVisible = dlg.m_bVisible;

		m_FileList[nIndex].bInclude = (dlg.m_nInclude == 1);
		m_FileList[nIndex].ConfigList = dlg.m_ConfigList;
		m_FileList[nIndex].Filename = dlg.m_RelativePath;
		m_FileList[nIndex].HPName = dlg.m_CompiledName;

		UpdateListControlItem(nIndex);

		SetModifiedFlag(TRUE);
	}

}

BOOL CProjectDocument::FileInCurrentConfig(int nIndex)
{
	BOOL bInclude;
	CString Space = _T(" ");
	CString IncludeList = Space + m_FileList[nIndex].ConfigList + Space;

	bInclude = !m_FileList[nIndex].bInclude;
	if (IncludeList.Find(Space + GetCurrentConfigName() + Space) != -1)
		bInclude = !bInclude;
	bInclude = bInclude && m_FileList[nIndex].bCompile;
	
	return bInclude;
}
