// ProjectEditView.cpp : implementation file
//

#include "stdafx.h"
#include "HPIDE.h"
#include "ProjectEditView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CProjectEditView

IMPLEMENT_DYNCREATE(CProjectEditView, CListView)

CProjectEditView::CProjectEditView()
{
	m_uiTimer = 0;
	m_pImageList = NULL;
}

CProjectEditView::~CProjectEditView()
{
	if (m_pImageList != NULL)
		delete m_pImageList;
}


BEGIN_MESSAGE_MAP(CProjectEditView, CListView)
	//{{AFX_MSG_MAP(CProjectEditView)
	ON_WM_CREATE()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_TIMER()
	ON_WM_RBUTTONDOWN()
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_PROJECT_MOVEFILEDOWN, OnProjectMovefiledown)
	ON_COMMAND(ID_PROJECT_MOVEFILEUP, OnProjectMovefileup)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CProjectEditView drawing

/////////////////////////////////////////////////////////////////////////////
// CProjectEditView diagnostics

#ifdef _DEBUG
void CProjectEditView::AssertValid() const
{
	CListView::AssertValid();
}

void CProjectEditView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CProjectEditView message handlers

int CProjectEditView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CListView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	CListCtrl& ListCtrl = GetListCtrl();

	m_pImageList = new CImageList;
	m_pImageList->Create(IDB_COMPILEFLAG, 16, 0, COLORREF(255, 255, 255));
	ListCtrl.SetImageList(m_pImageList, LVSIL_SMALL);
	ListCtrl.InsertColumn(0, _T("Filename"), LVCFMT_LEFT, 300);
	ListCtrl.InsertColumn(1, _T("Visible?"), LVCFMT_LEFT, 70);
	ListCtrl.InsertColumn(2, _T("Compile?"), LVCFMT_LEFT, 70);
	ListCtrl.InsertColumn(3, _T("Compiled name"), LVCFMT_LEFT, 100);

	m_uiTimer = SetTimer(0, 500, NULL);

	return 0;
}

BOOL CProjectEditView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	dwStyle |= LVS_REPORT;	
	return CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
}

void CProjectEditView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	CListView::OnLButtonDblClk(nFlags, point);
	CListCtrl& ListCtrl = GetListCtrl();
	int nItem = -1;
	for (int i = 0; i < ListCtrl.GetItemCount(); i++)
	{
		if (ListCtrl.GetItemState(i, LVIS_SELECTED) == LVIS_SELECTED)
		{
			nItem = i;
			break;
		}
	}
	if (nItem != -1)
	{
		
		CString Filename = ((CProjectDocument*)GetDocument())->m_FileList[nItem].Filename;
		if (Filename.Find(":") == -1 && Filename.Find("\\\\") == -1)
		{
			CString ProjectFile = GetDocument()->GetPathName();
			CString ProjectPath;
			char* pFilename;
			char* pPath = ProjectPath.GetBuffer(MAX_PATH);
			GetFullPathName(ProjectFile, MAX_PATH, pPath, &pFilename);
			ProjectPath.ReleaseBuffer();
			ProjectPath = ProjectPath.Left(pFilename - pPath);
			AfxGetApp()->OpenDocumentFile(ProjectPath + Filename);
		}
		else
		{
			AfxGetApp()->OpenDocumentFile(Filename);
		}
	}
}

void CProjectEditView::OnTimer(UINT nIDEvent) 
{
	CString Title;
	Title = GetDocument()->GetTitle();
	if (GetDocument()->IsModified())
	{
		if (Title.Right(2) != " *")
			Title += " *";
	}
	else
	{
		if (Title.Right(2) == " *")
			Title = Title.Left(Title.GetLength() - 2);
	}
	GetDocument()->SetTitle(Title);
	CListView::OnTimer(nIDEvent);
	m_uiTimer = SetTimer(0, 500, NULL);
}

void CProjectEditView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CListCtrl& ListCtrl = GetListCtrl();
	int nIndex = ListCtrl.HitTest(point, NULL);
	if (nIndex != -1)
	{
		for (int i = 0; i < ListCtrl.GetItemCount(); i++)
		{
			ListCtrl.SetItemState(i, 0, LVIS_SELECTED);
		}
		ListCtrl.SetItemState(nIndex, LVIS_SELECTED, LVIS_SELECTED);
	}
/*	CMenu menu;
	VERIFY(menu.LoadMenu(IDR_PROJECT_POPUP));
	CMenu* pPopup = menu.GetSubMenu(0);
	ASSERT(pPopup != NULL);
	ClientToScreen(&point);
	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, AfxGetMainWnd());*/

	CListView::OnRButtonDown(nFlags, point);
}


void CProjectEditView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if (nChar == VK_DELETE)
	{	
		CListCtrl& ListCtrl = GetListCtrl();

		int nCount = ListCtrl.GetSelectedCount();

		if (nCount > 0)
		{
			BOOL bOkay;
			if (nCount == 1)
			{
				bOkay = (MessageBox("Permanently remove this item from the project?", 
					"Confirm", 
					MB_YESNO | MB_ICONQUESTION) == IDYES);
			}
			else
			{
				CString Message;
				Message.Format("Permanently remove these %i items from the project?", nCount);
				bOkay = (MessageBox(Message,
					"Confirm", 
					MB_YESNO | MB_DEFBUTTON2 | MB_ICONQUESTION) == IDYES);
			}
			if (bOkay)
			{
				int nItem = -1;
				BOOL bNoneFound = FALSE;
				do
				{
					nItem = ListCtrl.GetNextItem(nItem, LVNI_SELECTED);
					if (nItem == -1)
						bNoneFound = TRUE;
					else
					{
						ListCtrl.DeleteItem(nItem);
						((CProjectDocument*)GetDocument())->m_FileList.RemoveAt(nItem);
						nItem--;
					}
				}  
				while (!bNoneFound);

				GetDocument()->SetModifiedFlag(TRUE);
			}
		}

	}
	CListView::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CProjectEditView::OnProjectMovefiledown() 
{
	CListCtrl& ListCtrl = GetListCtrl();

	BOOL bFoundNonSelected = FALSE;
	for (int i = ListCtrl.GetItemCount() - 1; i >= 0; i--)
	{
		if (ListCtrl.GetItemState(i, LVIS_SELECTED))
		{
			if (bFoundNonSelected)
			{
				CProjectDocument * pDoc = (CProjectDocument*)GetDocument();
				pDoc->MoveItem(i, i + 1);
				pDoc->UpdateListControlItem(i);
				pDoc->UpdateListControlItem(i + 1);
				ListCtrl.SetItemState(i + 1, LVIS_SELECTED, LVIS_SELECTED);
				ListCtrl.SetItemState(i, 0, LVIS_SELECTED);
				GetDocument()->SetModifiedFlag(TRUE);
			}
		}
		else
		{
			bFoundNonSelected = TRUE;
		}
	}
}

void CProjectEditView::OnProjectMovefileup() 
{
	CListCtrl& ListCtrl = GetListCtrl();

	BOOL bFoundNonSelected = FALSE;
	for (int i = 0; i < ListCtrl.GetItemCount(); i++)
	{
		if (ListCtrl.GetItemState(i, LVIS_SELECTED))
		{
			if (bFoundNonSelected)
			{
				CProjectDocument * pDoc = (CProjectDocument*)GetDocument();
				pDoc->MoveItem(i, i - 1);
				pDoc->UpdateListControlItem(i);
				pDoc->UpdateListControlItem(i - 1);
				ListCtrl.SetItemState(i - 1, LVIS_SELECTED, LVIS_SELECTED);
				ListCtrl.SetItemState(i, 0, LVIS_SELECTED);
				GetDocument()->SetModifiedFlag(TRUE);
			}
		}
		else
		{
			bFoundNonSelected = TRUE;
		}
	}
}
