// SourceEditView.cpp : implementation file
//

#include "stdafx.h"
#include "HPIDE.h"
#include "SourceEditView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSourceEditView

IMPLEMENT_DYNCREATE(CSourceEditView, CEditView)

CSourceEditView::CSourceEditView()
{
}

CSourceEditView::~CSourceEditView()
{
}


BEGIN_MESSAGE_MAP(CSourceEditView, CEditView)
	//{{AFX_MSG_MAP(CSourceEditView)
	ON_WM_KEYUP()
	ON_WM_KILLFOCUS()
	ON_WM_LBUTTONUP()
	ON_WM_SETFOCUS()
	ON_WM_TIMER()
	ON_WM_CHAR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSourceEditView diagnostics

#ifdef _DEBUG
void CSourceEditView::AssertValid() const
{
	CEditView::AssertValid();
}

void CSourceEditView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSourceEditView message handlers

BOOL CSourceEditView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext) 
{
	// Turn on scroll bars
	dwStyle |= WS_VSCROLL; 
	dwStyle |= WS_HSCROLL;
	// Turn on auto-scroll
	dwStyle |= ES_AUTOHSCROLL;
	dwStyle |= ES_AUTOVSCROLL;
	// Turn on multi-line
	dwStyle |= ES_MULTILINE;
	// Don't hide selection when don't have focus
	dwStyle |= ES_NOHIDESEL;
	BOOL bRet = CWnd::Create(lpszClassName, lpszWindowName, dwStyle, rect, pParentWnd, nID, pContext);
	if (bRet)
	{
		SendMessage(WM_SETFONT, (WPARAM)HFONT(((CHPIDEApp*)AfxGetApp())->m_SourceFont), TRUE);
	}

	m_uiTimer = SetTimer(0, 500, NULL);
	GetEditCtrl().SetMargins(15, 0);
	SetTabStops(8);
	return bRet;
}

void CSourceEditView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CEditView::OnKeyUp(nChar, nRepCnt, nFlags);
	UpdateLocationIndicator();
}

void CSourceEditView::OnKillFocus(CWnd* pNewWnd) 
{
	CEditView::OnKillFocus(pNewWnd);
	((CMainFrame*)AfxGetMainWnd())->m_wndStatusBar.GetStatusBarCtrl().SetText("", 1, 0);
}

void CSourceEditView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CEditView::OnLButtonUp(nFlags, point);
	UpdateLocationIndicator();
}

void CSourceEditView::UpdateLocationIndicator()
{
	int nStart, nEnd;
	GetEditCtrl().GetSel(nStart, nEnd);
	int nLine = GetEditCtrl().LineFromChar(nEnd);
	int nColStart = GetEditCtrl().LineIndex(nLine);
	CString LineBuffer;
	int nCount = GetEditCtrl().GetLine(nLine, LineBuffer.GetBuffer(nEnd - nColStart + 1), nEnd - nColStart);
	LineBuffer.ReleaseBuffer(nCount);
	int nCol = 1;
	for (int i = 0; i < LineBuffer.GetLength(); i++)
	{
		if (LineBuffer[i] == 0x09 && (nCol & 1 == 1))
			nCol++;
		nCol++;
	}
	CString LocationText;
	LocationText.Format("Ln: %i Col: %i", nLine + 1, nCol);
	((CMainFrame*)AfxGetMainWnd())->m_wndStatusBar.GetStatusBarCtrl().SetText(LocationText, 1, 0);
	
/*	int nColStart = GetEditCtrl().LineIndex(nLine);
	CString LocationText;
	LocationText.Format("Ln: %i Col: %i", nLine + 1, nEnd - nColStart + 1);
	((CMainFrame*)AfxGetMainWnd())->m_wndStatusBar.GetStatusBarCtrl().SetText(LocationText, 1, 0);*/
}

void CSourceEditView::OnSetFocus(CWnd* pOldWnd) 
{
	CEditView::OnSetFocus(pOldWnd);
	UpdateLocationIndicator();
}

void CSourceEditView::SetErrorLine(int nLine)
{
	int nPos = GetEditCtrl().LineIndex(nLine - 1);
	GetEditCtrl().SetSel(nPos, GetEditCtrl().LineIndex(nLine) - 1);
}

void CSourceEditView::OnTimer(UINT nIDEvent) 
{
	CString Title;
	Title = GetDocument()->GetTitle();
	if (GetDocument()->IsModified())
	{
		if (Title.Right(2) != " *")
			Title += " *";
	}
	else
	{
		if (Title.Right(2) == " *")
			Title = Title.Left(Title.GetLength() - 2);
	}
	GetDocument()->SetTitle(Title);
	CEditView::OnTimer(nIDEvent);
	m_uiTimer = SetTimer(0, 500, NULL);	
}

void CSourceEditView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	int nStart, nEnd;
	GetEditCtrl().GetSel(nStart, nEnd);
	int nLine = GetEditCtrl().LineFromChar(nEnd);
	int nColStart = GetEditCtrl().LineIndex(nLine);
	int nCol = nEnd - nColStart;
	switch (nChar)
	{
	case 0x0d: // Enter
		{
			CEditView::OnChar(nChar, nRepCnt, nFlags);
			CString LineBuffer;
			for (int nLineCheck = nLine; nLineCheck >= 0 && nLineCheck >= nLine - 5; nLineCheck--)
			{
				int nCopied = GetEditCtrl().GetLine(nLineCheck, LineBuffer.GetBuffer(1024), 1024);
				LineBuffer.ReleaseBuffer(nCopied);
				int nLength = LineBuffer.GetLength();
				for (int i = 0; i < nLength; i++)
				{
					if (LineBuffer[i] != 0x09)
						break;
				}
				if (nLength && LineBuffer[0] != 0x09)
					break;
				if (i != nLength && i != 0)
				{
					for (int j = 0; j < i; j++)
						SendMessage(WM_CHAR, 0x09, 1);
					break;
				}

			}
		}
	break;
	default:
		{
			CEditView::OnChar(nChar, nRepCnt, nFlags);
		}
	}
}
